/* LICENSE: no restrictions at all, but USE ON YOUR OWN RISK ONLY.
 * SPDX-License-Identifier: LicenseRef-Wrobel
 * Copyright 2004-2008 Angela Wrobel <http://www.wrobelnet.de/>, et. al.
 */
#ifndef __STDINT_H
#define __STDINT_H
#include <sys/types.h>
typedef  unsigned char   uint8_t;
typedef  unsigned short  uint16_t;
typedef  unsigned long   uint32_t;

typedef    signed char   int8_t;
typedef    signed short  int16_t;
typedef    signed long   int32_t;
#endif


